// ATX Trespasser 2.00+ Plugin System
// Main Include File (for communication with ATYPEX.dll)
// Requires windows definitions and "atxtresclassdefs.h"
// Do not modify this file when writing plugins, unless performing simple translation.
//
// Notes:
// - All structures must be aligned to 4 bytes (most entries are already DWORD sized)
// - the uint type is an unsigned int that MUST be 32-bits (must be compiled for 32-bit x86 machine)


#ifndef atxpluginH
#define atxpluginH


// required includes

#include <windows.h>

// import/export call types

#define atxcallback __stdcall
#define atximport __stdcall


// simple type definitions

typedef unsigned int	uint;
typedef uint		TRESEXETYPE;
typedef uint		ID32;

// constants

#define DDSurface			void
#define DDObj				void


// Trespasser ATX executable ID's

#define TRES_EXE_P611			0x31313650
#define TRES_EXE_P511			0x31313550
#define TRES_EXE_K611			0x3131364B
#define TRES_EXE_DEMO			0x4F4D4544


// console constants

#define MAX_CONSOLE_CHARS		32


// player AIType function constants

#define AITYPE_PLAYER			5
#define AITYPE_INVISIBLE		0
#define AITYPE_TOGGLE			-1


// player CGun function constants

#define GETWEAPON_PRIMARY		0x0E34
#define GETWEAPON_SECONDARY		-2	// there are two possible locations for secondary weapon
#define GETWEAPON_SECONDARYWAIST	0x0E38
#define GETWEAPON_SECONDARYBACK 	0x0E3C


// new ActionType constants

#define TSCRIPT_VALUETYPE_BOOL		1
#define TSCRIPT_VALUETYPE_INT		3
#define TSCRIPT_VALUETYPE_FLOAT 	4
#define TSCRIPT_VALUETYPE_STRING	5
#define TSCRIPT_VALUETYPE_GROUP 	6

#include "atxtscriptindexes.h"

// simple Trespasser/ATX structures

typedef struct SObjLoc {
	       float x;
	       float y;
	       float z;
	} SObjLoc;

typedef struct SQuaternion {
	       float w;
	       float x;
	       float y;
	       float z;
	} SQuaternion;

typedef struct SObjLocOrient {
	       SObjLoc objLoc;
	       SQuaternion objOrient;
	} SObjLocOrient;

typedef struct SObjState {
	       SObjLoc objLoc;
	       SQuaternion objOrient;
	       float scale;
	} SObjState;

typedef struct SPixel2Dz {
	       unsigned int x;
	       unsigned int y;
	       float zReciprocal;
	} SPixel2DwZ;

typedef struct SVelocity {
	       float x;
	       float y;
	       float z;
	} SVelocity;

typedef struct SRotVelocity {
	       float w;
	       float x;
	       float y;
	       float z;
	} SRotVelocity;

typedef struct SRotVel {
	       SVelocity vel;
	       SRotVelocity rotVel;
	} SRotVel;

typedef struct SVector3 {
	       float x;
	       float y;
	       float z;
	} SVector3;


typedef struct SAnimateHP {
	       float HitPoints;
	       float MaxHitPoints;
	} AnimateHP;


typedef struct SDimensions2 {
	       uint X;
	       uint Y;
	} SDimensions2;

// new plugin ActionType function types

typedef void atxcallback newatdestp(void *);
typedef void atxcallback newatmainp(void *);
typedef void atxcallback newatreinp(void *);


#include "atxtresclassdefs.h"

// function declarations

#ifdef __cplusplus
extern "C" {
#endif


// exportable ATX callback function declarations

void atxcallback ATXCPostDLLLoad(const char *pluginDLLPath,const char *pluginFolderPath);
void atxcallback ATXCPreDLLUnload();
void atxcallback ATXCPreScreenRender();
void atxcallback ATXCPostScreenRender();
void atxcallback ATXCPreLevelLoad(bool isNewGRFLoad);
void atxcallback ATXCPostLevelLoad(bool isNewGRFLoad);
void atxcallback ATXCOnActBite(CActivityBite *biteInst, CInstance *biteTarget, CInfluence *sourceInfluence);
void atxcallback ATXCOnLevelSave(const char *saveFilePathName);
void atxcallback ATXCOnPlgUserCommand(const char *args);
void* atxcallback ATXCNewATInit(uint atNum, CObjectValue *lpSourceObject);


// importable ATX function declarations

void atximport ATXCCameraTeleport(const SObjLocOrient *newLoc);
void atximport ATXChangePlayerAIType(uint newAIType);
bool atximport ATXConvert3DPixelTo2DRender(SPixel2Dz *lpResult, const SObjLoc *pixel3DLoc);
void atximport ATXDisableInGameKeys();
void atximport ATXDisableInGameMvtKeys();
void atximport ATXEnableInGameKeys();
void atximport ATXFindNextSaveGameFileName(char *lpResult);
void atximport ATXGameMemDelete(void *memToFree);
void* atximport ATXGameMemNew(uint numberOfBytes);
bool atximport ATXGetAllCAnimalInstances(CAnimal **lpAnimalsTable);
const SAnimateHP* atximport ATXGetAnimateHealth(CAnimate *lpInst);
HDC atximport ATXGetBackBufferSurfaceDC();
CAISystem* atximport ATXGetCAISystemInstance();
uint atximport ATXGetCAnimalInstanceCount();
CCamera* atximport ATXGetCCameraInstance();
CParticles* atximport ATXGetCParticlesFromID32(ID32 particlesID);
ID32 atximport ATXGetCParticlesID32FromString(const char *lpTypeName);
CPlayerPriv* atximport ATXGetCPlayerPrivInstance();
CAnimal* atximport ATXGetCurrentBitingAnimal();
DDObj* atximport ATXGetDDrawObjectInGame();
DDSurface* atximport ATXGetDDrawSurfaceCurrent();
const SDimensions2* atximport ATXGetDDScreenDimensions();
uint atximport ATXGetDDSurfaceStructSize();
TRESEXETYPE atximport ATXGetEXEType();
uint atximport ATXGetGameDifficulty();
const float* atximport ATXGetGameSpeed();
const float* atximport ATXGetGravity();
CGun* atximport ATXGetHeldWeaponInstance(uint weaponLocation);
float atximport ATXGetINIFloatEntry(const char *sectionName, const char *keyName, const char *defaultVal, const char *iniPath);
CInstance* atximport ATXGetInstanceFromID32(ID32 instanceID);
CInstance* atximport ATXGetInstanceFromString(const char *instanceName);
CInstance* atximport ATXGetInstanceFromStringCBD(const char *instanceName);
ID32 atximport ATXGetInstanceID32(CInstance *lpInstance);
ID32 atximport ATXGetInstanceID32FromString(const char *lpInstName);
const SObjState* atximport ATXGetInstanceLocation(CInstance *lpInst);
bool atximport ATXGetInstancePhysicsVelocity(SRotVel *lpVelocity, const CInstance *lpInstance);
const char* atximport ATXGetLevelSavedSceneName();
uint atximport ATXGetPlayerAIType();
uint atximport ATXGetTimeSinceLastLevelLoad();
void* atximport ATXNewATAllocInstance(uint memSize, newatdestp *lpDestructor, newatmainp *lpMainProc, newatreinp *lpReInitProc);
void atximport ATXNewATFreeInstance(void *lpThis);
bool atximport ATXNewATGetGroupValue(CObjectValue **lpFarReturnGroupObject, uint valueNameIndex, CObjectValue *lpSourceObject, uint *lpEntryCount);
void* atximport ATXNewATGetScriptValue(uint valueNameIndex, CObjectValue *lpSourceObject, uint entryNum, uint valueType);
CGun* atximport ATXIsPlayerHoldingGun();
bool atximport ATXIsValidInstance(void *lpMem, uint memSize);
void atximport ATXReleaseBackBufferSurfaceDC(HDC bufHandle);
void atximport ATXResetCCameraPlayerSources();
void atximport ATXSeedParticleEffect(CParticles *lpEffect, SObjLoc *lpLocation, SVector3 *lpDirection, float param7D, float pThreshold);
void atximport ATXSetAnimateHealth(CAnimate *lpInstance, float newHealth);
void atximport ATXSetAnimateMaxHealth(CAnimate *lpInstance, float newMaxHealth);
void atximport ATXSetGameSpeed(float newGameSpeed);
void atximport ATXSetInstanceLocation(CInstance *lpInstance, SObjLocOrient *lpLoc);
void atximport ATXSetInstancePhysicsUnfreezeOnly(CInstance *lpInstance);
void atximport ATXSetInstancePhysicsVelocity(CInstance *lpInstance, const SRotVel *lpVelocity);
void atximport ATXTestExport(const char *testMessage);
void atximport ATXWriteLogFile(const char *text, uint textSize);


#ifdef __cplusplus
}
#endif


#endif