;============================================================
ATXPROC FIXOBJECTSPEED_RT
;============================================================
; FIX OBJECT SPEED AT110 REAL-TIME IMPLEMENTATION
; always called if main buffer is non-zero

push ebp

	invoke GetTickCount
	sub eax,[FixSLastTickGlobal]
	mov ebp,eax							; tick difference between frames

	mov esi,[FixSMainBuffer]
	mov ebx,[FixSMainBufferSizeDiv2]
	xor eax,eax

	.mainloop:
	cmp [esi],eax
	mov edi,[esi+ebx]
	jz .noinstanceid

		mov eax,[edi+AT110_PROJ_INFO.TickLeft]
		cmp eax,-1
		jz .instancetimeleft
		sub eax,ebp
		ja .instancetimeleft

			mov DWORD [esi],0				; erase instance name ID
			invoke LocalFree,[esi+ebx]			; free structure
			jmp .noinstanceid

		.instancetimeleft:

		mov [edi+AT110_PROJ_INFO.TickLeft],eax			; update remaining tick time

	 ; set instance velocity via ActionType 10

		mov ecx,AT10SimObjectVelOnly
		mov eax,[edi+AT110_PROJ_INFO.Instance]
		mov edx,[edi+AT110_PROJ_INFO.Velocity.X]
		mov [ecx+CSetPhysicsAction.TargetInstance],eax
		mov [ecx+CSetPhysicsAction.Velocity.X],edx
		mov eax,[edi+AT110_PROJ_INFO.Velocity.Y]
		mov edx,[edi+AT110_PROJ_INFO.Velocity.Z]
		mov [ecx+CSetPhysicsAction.Velocity.Y],eax
		mov [ecx+CSetPhysicsAction.Velocity.Z],edx

		mov edx,[ecx]
		mov [ecx+CSetPhysicsAction.RefCount],0
		mov [ecx+CSetPhysicsAction.Facing.Y],1.000000
		call [edx+ICAction.MainProc]				; finally, set speed at every frame

	.noinstanceid:
	add esi,4
	xor eax,eax
	cmp esi,[FixSMainBufferHalfEnd]
	jb .mainloop

	add [FixSLastTickGlobal],ebp

pop ebp
ret


;============================================================
ATXPROC FIXOBJECTSPEED_INIT
;============================================================
; buffer released in PRELEVELLOAD.inc

push ebp
push ebx
push esi
push edi

	 mov eax,[FixSNumberOfAT110Instances]
	 add eax,[FixSSPTProjectileCount]
	 shl eax,2
	 mov [esp-4],eax
	 shl eax,1
	 test eax,eax
	 jz .forceexit

	 sub esp,4
	 invoke LocalAlloc,LPTR,eax
	 test eax,eax
	 pop ecx
	 jz .forceexit

	 mov [FixSMainBuffer],eax					; possible previous buffers released in PRELEVELLOAD.inc
	 mov ebx,eax
	 mov [FixSMainBufferSizeDiv2],ecx
	 add ecx,eax
	 mov [FixSMainBufferHalfEnd],ecx

   ; If we loaded a savegame containing a saved buffer,
   ; initialize its contents and release.
   ; format is (following 8 first bytes):
   ; DWORD (instance name ID), structure, DWORD (instance name ID), structure, etc.

	 mov esi,[FixSSaveDataTempBuf]
	 mov edi,[FixSSaveDataTempBUFSIZE]
	 test esi,esi
	 jz .initloopend
	 cmp edi,(5*4+8+4)
	 jb .initloopend
	 mov ebp,[esi+04]						; struc size
	 cmp edi,ebp							; at least this size required (one instance name + struc)
	 jb .initloopend
	 cmp DWORD [esi],152						; currently, this is the only existing AT110 version
	 jnz .initloopend

	 add edi,esi
	 add esi,8

.initloop:

	 GETINSTANCEFROMID32 DWORD [esi]
	 test eax,eax
	 jz .noinstance 						; is the instance still valid in this level? (should always be)

	 push eax							; save instance itself
	 invoke LocalAlloc,LPTR,sizeof.AT110_PROJ_INFO			; allocate space for structure (all members NULL by default)
	 test eax,eax
	 pop edx
	 jz .initloopend

		push eax
		push esi
		push edi

		mov edi,eax
		lea esi,[esi+4]
		mov ecx,ebp
		rep movsb						; copy structure to new buffer

		pop edi
		pop esi
		pop eax
		mov ecx,[FixSMainBufferSizeDiv2]
		mov [eax+AT110_PROJ_INFO.Instance],edx			; set instance in struct
		mov [ebx+ecx],eax					; store structure in final buffer
		mov ecx,[esi]						; get instance ID again
		mov [ebx],ecx						; store instance ID as valid

	 .noinstance:

	 add ebx,4
	 cmp ebx,[FixSMainBufferHalfEnd]				; are we out of space? (should not happen)
	 jnb .initloopend

	 lea esi,[esi+ebp+4]
	 cmp esi,edi
	 jb .initloop

.initloopend:

	 LOCALRELEASE [FixSSaveDataTempBuf]
	 mov [FixSSaveDataTempBUFSIZE],0

	 invoke GetTickCount
	 mov [FixSLastTickGlobal],eax

pop edi
pop esi
pop ebx
pop ebp
ret


.forceexit:

	 call ReleaseFIXOBJECTSPEEDmainbuffer

	 LOCALRELEASE [FixSSaveDataTempBuf]
	 mov [FixSSaveDataTempBUFSIZE],0

pop edi
pop esi
pop ebx
pop ebp
ret


;============================================================
ATXPROC ReleaseFIXOBJECTSPEEDmainbuffer
;============================================================
push esi
push edi
push ebx

	mov esi,[FixSMainBuffer]
	mov edi,[FixSMainBufferSizeDiv2]
	test esi,esi
	jz .noreleasemain

	mov ebx,[FixSMainBufferHalfEnd]

	.releaseloop:
	cmp DWORD [esi],0
	jz @f

		invoke LocalFree,[esi+edi]

	@@:
	add esi,4
	cmp esi,ebx
	jb .releaseloop

	LOCALRELEASE [FixSMainBuffer]

	.noreleasemain:
	xor eax,eax
	mov [FixSMainBufferSizeDiv2],eax
	mov [FixSMainBufferHalfEnd],eax

pop ebx
pop edi
pop esi
ret
