;====================================================================
ATINITPROC _AT114Init
;====================================================================
; fetches group listing for A00 main group (holds instance names to teleport)

; notes:
;     - does not keep track of counters for main directory (which holds
;       int ActionType = 114 entry). Only keeps track of counter for one
;       sub-group at a time (more than one not necessary due to groups being
;       scanned only one at a time).
;
;     - In this code, LOCATION GROUP refers to the SUB-GROUPs in
;       the documentation, while SUB-GROUPS refer to the LOCATION groups
;       in the documentation (interchanged for the sake of confusion)

	xor ebx,ebx								; ebx = buffer for group pointers
	xor edi,edi								; edi = official AT114 instance
	mov [AT114MAINGROUPCounter],ebx
	mov [AT114TotalProbabilityCount],ebx
	mov [AT114ExtraAllocBytes],ebx
	mov [AT114TotalAllocations],ebx
	mov [AT114TotalAllocationsSize],ebx


; switch to MAIN GROUP (A00)

	mov ecx,AT114SUBGROUPCounter1
	mov edx,AT114SUBGROUPIDValue1
	mov eax,[ebp-28]
	mov DWORD [ecx],0
	add eax,TSCRIPT_OFFSETVALUE_A00 					; table displacement for the T-Script value

	cinvoke GetGroupIDValueProc,edx,eax,[ebp-32],[ebp-36],ecx		; returns al=false if not found/error

	mov edx,[AT114SUBGROUPIDValue1]
	test al,al
	mov [AT114LastError],AT114_ERRORSTR_NOMAINGROUP
	jz .noinitialvalues

	test edx,edx
	jz .noinitialvalues						   ; group value exists?

; get int Frame value = number of groups

	mov eax,[ebp-28]
	add eax,TSCRIPT_OFFSETVALUE_FRAME					; first entry in group

	cinvoke GetEntryValueProc,eax,edx,[ebp-36],[AT114SUBGROUPCounter1]

	test eax,eax
	mov [AT114LastError],AT114_ERRORSTR_INVALIDFRAMEVALUE
	mov [AT114LastErrorDParam1],0
	jz .noinitialvalues

	cmp DWORD [eax+4],TSCRIPT_VALUETYPE_INT
	jnz .noinitialvalues

	mov edx,[eax+28h]							; value of Frame
	cmp edx,0
	mov [AT114LastErrorDParam1],edx
	    jle .noinitialvalues

	cmp edx,AT114_GROUPCOUNTLIMIT						; number of GROUP (Axx) values should be between 1 and 199 inclusively
	    jnb .noinitialvalues

	mov [AT114NumberOfGroups],edx
	inc [AT114SUBGROUPCounter1]

; get int Sample value = number of Axx instances (don't want to scan all for nothing)

	mov edx,[AT114SUBGROUPIDValue1]
	mov eax,[ebp-28]
	add eax,TSCRIPT_OFFSETVALUE_SAMPLE

	cinvoke GetEntryValueProc,eax,edx,[ebp-36],[AT114SUBGROUPCounter1]

	test eax,eax
	mov [AT114LastError],AT114_ERRORSTR_INVALIDSAMPLEVALUE
	mov [AT114LastErrorDParam1],0
	jz .noinitialvalues

	cmp DWORD [eax+4],TSCRIPT_VALUETYPE_INT
	jnz .noinitialvalues

	mov edx,[eax+28h]							; value of Sample
	cmp edx,0
	mov [AT114LastErrorDParam1],0
	jle .noinitialvalues

	cmp edx,200								; number of Axx values should be between 1 and 199 inclusively
	ja .noinitialvalues

	mov [AT114NumberOfInstances],edx
	inc [AT114SUBGROUPCounter1]

; fetch instances to teleport

	mov esi,1

	.getinstancesloop:

	mov edx,[AT114SUBGROUPIDValue1]
	mov eax,[ebp-28]
	lea eax,[eax+8*esi+TSCRIPT_OFFSETVALUE_A00]

	cinvoke GetEntryValueProc,eax,edx,[ebp-36],[AT114SUBGROUPCounter1]

	test eax,eax
	mov [AT114LastError],AT114_ERRORSTR_MISSINGINSTANCE
	mov [AT114LastErrorDParam1],esi
	jz .noinitialvalues

	cmp DWORD [eax+4],TSCRIPT_VALUETYPE_STRING
	jnz .noinitialvalues

	inc [AT114SUBGROUPCounter1]
	mov eax,[eax+40h]							; get string name pointer
	test eax,eax
	jz .noinitialvalues

	GETNAMEID32STD eax
	test eax,eax
	jz .noinitialvalues

	mov [AT114InstanceIDBuffer+4*esi-4],eax
	inc esi
	cmp esi,[AT114NumberOfInstances]
	jbe .getinstancesloop


; create main temporary buffer, holds pointers for all groups (to location info structures)
;        ebx = temporary buffer
;        size = 5*(number of groups)*(number of groups)
;        (first part = series of BYTE flags, second half = series of DWORD pointers)

	xor ebx,ebx
	mov eax,[AT114NumberOfGroups]
	mov ecx,[AT114NumberOfInstances]
	mul ecx
	mov [AT114NumberOfGroupsXInstances],eax
	lea eax,[eax+eax*4]
	mov ecx,[AT114TotalAllocationsSize]
	mov [AT114TempBufferSize],eax
	mov [AT114LastErrorDParam1],eax
	mov [AT114LastErrorDParam2],AT114_ERRORSTR_NOLOCALMEM_MAINTEMP
	mov [AT114LastErrorDParam3],0
	add ecx,eax
	mov [AT114TotalAllocationsSize],ecx
	mov [AT114LastErrorDParam4],ecx

	invoke LocalAlloc,LMEM_FIXED,eax
	test eax,eax
	mov [AT114LastError],AT114_ERRORSTR_NOLOCALMEM
	jz .noinitialvalues

	inc [AT114TotalAllocations]
	mov ebx,eax

; set initial flags for all groups to none

	push edi

	mov ecx,[AT114NumberOfGroupsXInstances]
	mov edi,ebx
	ZEROMEM 								; default flags = 0

; set initial group probability values to zero

	mov ecx,[AT114NumberOfGroups]
	xor eax,eax
	mov edi,AT114GroupProbabilities
	rep stosd


; set initial location sets flags to zero (NEW IN ATX 2.09)

	mov ecx,[AT114NumberOfGroups]
	mov edi,AT114GroupFlags
	ZEROMEM

	pop edi

; MAIN GROUPS FETCH LOOP

	mov [AT114SUBGROUPCountCurrent],1
	mov esi,ebx
	mov [AT114FlagsPointerCurrent],ebx					; this mem is storage for location entry flags
	add esi,[AT114NumberOfGroupsXInstances] 				; esi points to storage for location entry pointers

	.getgroupsloop:

; get next SUB-GROUP 1

	mov edx,[AT114SUBGROUPCountCurrent]
	mov ecx,AT114SUBGROUPCounter1
	mov eax,[ebp-28]
	mov DWORD [ecx],0
	lea eax,[eax+edx*8+TSCRIPT_OFFSETVALUE_A00]

	cinvoke GetGroupIDValueProc,AT114SUBGROUPIDValue1,eax,[ebp-32],[ebp-36],ecx

	test al,al
	mov edx,[AT114SUBGROUPIDValue1]
	jz .nogroup

	test edx,edx
	jz .nogroup

	; get LoopCount probability value from this SUB-GROUP

	     mov eax,[ebp-28]
	     add eax,TSCRIPT_OFFSETVALUE_LOOPCOUNT

	     GETVALUEEXT eax,[AT114SUBGROUPIDValue1],DWORD [ebp-36],\
			 AT114SUBGROUPCounter1,TSCRIPT_VALUETYPE_INT

	     test ecx,ecx							; was entry valid
	     jz .nogroup
	     cmp eax,0								; is value zero or negative?
	     jle .nogroup
	     add [AT114TotalProbabilityCount],eax

	; save probability count for this group

	     mov ecx,[AT114SUBGROUPCountCurrent]
	     mov [AT114GroupProbabilities+ecx*4-4],eax

	; get randomization boolean for this SUB-GROUP (NEW IN ATX 2.09)

	     mov eax,[ebp-28]
	     add eax,TSCRIPT_OFFSETVALUE_R

	     GETVALUEEXT eax,[AT114SUBGROUPIDValue1],DWORD [ebp-36],\
			 AT114SUBGROUPCounter1,TSCRIPT_VALUETYPE_BOOL

	     test ecx,ecx							; was entry valid
	     jz .norandboolset
	     test al,al
	     jz .norandboolset
	     mov eax,[AT114SUBGROUPCountCurrent]
	     or [AT114GroupFlags+eax*4-4],AT114_GROUP_RANDOMIZELOCATIONSINGROUP
	     .norandboolset:

	; SUB-LOOP LOCATION VALUES FETCH FOR CURRENT GROUP

	     mov [AT114LOCATIONGROUPCountCurrent],1
	     .getlocationssubloop:

	; get next sub-group containing location values

	     mov edx,[AT114LOCATIONGROUPCountCurrent]
	     mov ecx,AT114SUBGROUPCounter2
	     mov eax,[ebp-28]
	     mov DWORD [ecx],0
	     lea eax,[eax+edx*8+TSCRIPT_OFFSETVALUE_A00]

	     cinvoke GetGroupIDValueProc,AT114SUBGROUPIDValue2,eax,[AT114SUBGROUPIDValue1],[ebp-36],ecx

	     test al,al
	     mov edx,[AT114SUBGROUPIDValue2]
	     jz .getlocationssubloop_NoGroup

	     test edx,edx
	     jz .getlocationssubloop_NoGroup

	; set default flags for this location group (excludes enable flag)

	     mov edx,[AT114FlagsPointerCurrent]
	     mov BYTE [edx],AT114_LOC_USEDESTOBJECT+AT114_LOC_USEONTERRAIN+AT114_LOC_USESETORIENTATION+\
			    AT114_LOC_USESETPOSITION+AT114_LOC_USEHEIGHTRELATIVE	; defaults

	; check for SetOrientation bool

	     GETVALUEAT114FLAGS TSCRIPT_OFFSETVALUE_SETORIENTATION,[AT114SUBGROUPIDValue2],\
				[AT114SUBGROUPCounter2],[AT114FlagsPointerCurrent],AT114_LOC_USESETORIENTATION

	; check for SetPosition bool

	     GETVALUEAT114FLAGS TSCRIPT_OFFSETVALUE_SETPOSITION,[AT114SUBGROUPIDValue2],\
				[AT114SUBGROUPCounter2],[AT114FlagsPointerCurrent],AT114_LOC_USESETPOSITION

	; check if position and orientation are set

	     mov edx,[AT114FlagsPointerCurrent]
	     test BYTE [edx],(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	     jz .getlocationssubloop_NoGroup					; group cannot have no location and no orientation set

	; check for HeightRelative bool

	     GETVALUEAT114FLAGS TSCRIPT_OFFSETVALUE_HEIGHTRELATIVE,[AT114SUBGROUPIDValue2],\
				[AT114SUBGROUPCounter2],[AT114FlagsPointerCurrent],AT114_LOC_USEHEIGHTRELATIVE


	; check for OnTerrain bool

	     GETVALUEAT114FLAGS TSCRIPT_OFFSETVALUE_ONTERRAIN,[AT114SUBGROUPIDValue2],\
				[AT114SUBGROUPCounter2],[AT114FlagsPointerCurrent],AT114_LOC_USEONTERRAIN

	; check for TeleportDestObjectName

	     mov edx,[AT114SUBGROUPIDValue2]
	     mov eax,[ebp-28]
	     add eax,TSCRIPT_OFFSETVALUE_TELEPORTDESTOBJECTNAME

	     cinvoke GetEntryValueProc,eax,edx,[ebp-36],[AT114SUBGROUPCounter2]

	     test eax,eax
	     jz .noteleportdestobjectname

	     cmp DWORD [eax+4],TSCRIPT_VALUETYPE_STRING
	     jnz .noteleportdestobjectname

	     inc [AT114SUBGROUPCounter2]
	     cmp DWORD [eax+28h],0
	     jle .noteleportdestobjectname

	     mov eax,[eax+40h]
	     test eax,eax
	     jz .noteleportdestobjectname

	     GETNAMEID32STD eax
	     test eax,eax
	     jz .noteleportdestobjectname

	     mov [esi],eax							; store string ID
	     jmp .groupfound

	     .noteleportdestobjectname:

	; since not a Teleport Dest Object, use X,Y,Z values

	     mov edx,[AT114FlagsPointerCurrent]
	     mov al,[edx]
	     and BYTE [edx],(not AT114_LOC_USEDESTOBJECT)
	     and al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	     mov ecx,(3*4)
	     add [AT114ExtraAllocBytes],ecx
	     cmp al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	     jnz @f

			mov ecx,(6*4)						; (last member is not immediately used)
			add [AT114ExtraAllocBytes],(4*4)			; count extra DWORD for quaternion conversion later

	     @@:

	; allocate temporary memory for values

	     mov eax,[AT114TotalAllocations]
	     mov edx,[AT114TotalAllocationsSize]
	     mov [AT114LastErrorDParam1],ecx
	     mov [AT114LastErrorDParam2],AT114_ERRORSTR_NOLOCALMEM_SMALLTEMP
	     mov [AT114LastErrorDParam3],eax
	     add edx,ecx
	     mov [AT114TotalAllocationsSize],edx
	     mov [AT114LastErrorDParam4],edx

	     invoke LocalAlloc,LMEM_FIXED,ecx
	     test eax,eax
	     mov [AT114LastError],AT114_ERRORSTR_NOLOCALMEM
	     jz .exitinit

	     inc [AT114TotalAllocations]
	     mov [esi],eax

	; get X,Y,Z values, if applicable

	     mov ecx,eax

	     mov [AT114LastError],AT114_ERRORSTR_INVALIDXYZ

	     mov eax,[AT114LOCATIONGROUPCountCurrent]
	     mov edx,[AT114SUBGROUPCountCurrent]
	     mov [AT114LastErrorDParam2],eax
	     mov [AT114LastErrorDParam3],edx					; (error values)

	     mov edx,[AT114FlagsPointerCurrent]
	     test BYTE [edx],AT114_LOC_USESETPOSITION
	     jz .nofetchpositionvalues

	     mov eax,[ebp-28]
	     add eax,TSCRIPT_OFFSETVALUE_X

	     stdcall GetTripletScriptValues,eax,[AT114SUBGROUPIDValue2],[ebp-36],\
		     AT114SUBGROUPCounter2,TSCRIPT_VALUETYPE_FLOAT,ecx

	     test eax,eax
	     mov [AT114LastErrorDParam1],AT114_ERRORSTR_INVALIDXYZ_XYZ
	     jz .exitinit						   ; or .getlocationssubloop_NoGroup

			mov ecx,[esi]
			add ecx,12

	     .nofetchpositionvalues:

	; get XFree, YFree, ZFree values, if applicable (will be converted to quaternion later, during memcopy)

	     mov edx,[AT114FlagsPointerCurrent]
	     test BYTE [edx],AT114_LOC_USESETORIENTATION
	     jz .groupfound

	     mov eax,[ebp-28]
	     add eax,TSCRIPT_OFFSETVALUE_XFREE

	     stdcall GetTripletScriptValues,eax,[AT114SUBGROUPIDValue2],[ebp-36],\
					    AT114SUBGROUPCounter2,TSCRIPT_VALUETYPE_FLOAT,ecx

	     test eax,eax
	     mov [AT114LastErrorDParam1],AT114_ERRORSTR_INVALIDXYZ_XYZFREE
	     jz .exitinit						   ; or .getlocationssubloop_NoGroup

	; set enable flag for this location group

	     .groupfound:
	     mov edx,[AT114FlagsPointerCurrent]
	     or BYTE [edx],AT114_LOC_ENTRYVALID

	; end of LOCATION GROUP loop

	     .getlocationssubloop_NoGroup:
	     mov eax,[AT114LOCATIONGROUPCountCurrent]
	     add esi,4
	     inc eax
	     inc [AT114FlagsPointerCurrent]
	     cmp eax,[AT114NumberOfInstances]
	     mov [AT114LOCATIONGROUPCountCurrent],eax
	     jbe .getlocationssubloop

; end of SUB GROUP loop

	.nogroup:

	mov eax,[AT114SUBGROUPCountCurrent]
	inc eax
	cmp eax,[AT114NumberOfGroups]
	mov [AT114SUBGROUPCountCurrent],eax
	jbe .getgroupsloop


; make sure total probability count is not zero

	cmp [AT114TotalProbabilityCount],0
	mov [AT114LastError],AT114_ERRORSTR_INVALIDPROBABILITY
	jz .exitinit

; allocate memory for main instance and organize
; structure:
; sizeof.CAT114
; 4*(number of instances)                                       = instance IDs
; 4*(number of groups)                                          = group probabilities list
; 1*(number of groups)                                          = group flags (NEW IN ATX 2.09)
; 1*(number of instances)*(number of groups)                    = BYTE flags for each Location entry (present or not)
; 4*(number of instances)*(number of groups)                    = series of pointers for each location entry
;                                                                 (pointers are either to x,y,z locations in next section
;                                                                 or holds pointer to some instance if DestObjectName used)
; (space reserved = size defined by [AT114_ExtraAllocBytes])    = holds x,y,z data pointed to for locations w/o dest.objects

	mov eax,[AT114NumberOfInstances]
	mov ecx,[AT114NumberOfGroups]
	shl eax,2
	shl ecx,2
	mov esi,sizeof.CAT114
	mov edx,[AT114NumberOfGroupsXInstances]
	add esi,eax
	add esi,ecx
	add esi,[AT114NumberOfGroups]						; NEW IN ATX 2.09
	lea eax,[edx*4]
	mov ecx,[AT114ExtraAllocBytes]
	add esi,edx
	add esi,eax								; esi will point to spare space for X,Y,Z values
	add ecx,esi

; allocate official instance memory

	mov [AT114InstanceSize],ecx

	mov eax,[AT114TotalAllocations]
	mov edx,[AT114TotalAllocationsSize]
	mov [AT114LastErrorDParam1],ecx
	mov [AT114LastErrorDParam2],AT114_ERRORSTR_NOLOCALMEM_MAINOBJ
	mov [AT114LastErrorDParam3],eax
	add edx,ecx
	mov [AT114TotalAllocationsSize],edx
	mov [AT114LastErrorDParam4],edx

	cinvoke YAPAXI,ecx

	test eax,eax
	mov edi,eax
	mov [AT114LastError],AT114_ERRORSTR_NOLOCALMEM
	jz .exitinit

	inc [AT114TotalAllocations]

	add esi,edi								; esi points to space for x,y,z data

	lea eax,[AT114]
	mov [edi+CAT114.lpVTable],eax
	mov [edi+CAT114.refCount],0
	mov eax,[AT114TotalProbabilityCount]
	mov ecx,[AT114NumberOfGroups]
	mov edx,[AT114NumberOfInstances]
	mov [edi+CAT114.TotalProbabilityCount],eax
	mov [edi+CAT114.NumberOfGroups],ecx
	mov [edi+CAT114.NumberOfInstances],edx

	lea ecx,[edi+sizeof.CAT114]
	mov eax,[AT114NumberOfInstances]
	mov edx,[AT114NumberOfGroups]
	shl eax,2
	shl edx,2
	mov [edi+CAT114.lpInstanceList],ecx
	add ecx,eax
	mov [edi+CAT114.lpGroupsProbabilitiesList],ecx
	add ecx,edx
	mov [edi+CAT114.lpGroupsFlags],ecx					; new in ATX 2.09
	add ecx,[AT114NumberOfGroups]						; ...
	mov [edi+CAT114.lpGroupLocationFlags],ecx
	add ecx,[AT114NumberOfGroupsXInstances]
	mov [edi+CAT114.lpGroupData],ecx

;copy instance ID, group probability, and group flags buffers

	push esi
	push edi

	push [edi+CAT114.lpGroupsFlags] 					; new in ATX 2.09
	push [edi+CAT114.lpGroupsProbabilitiesList]

	mov edi,[edi+CAT114.lpInstanceList]
	mov esi,AT114InstanceIDBuffer
	mov ecx,[AT114NumberOfInstances]
	rep movsd

	pop edi
	mov esi,AT114GroupProbabilities
	mov ecx,[AT114NumberOfGroups]
	rep movsd

	pop edi 								 ; /* new in ATX 2.09
	mov esi,AT114GroupFlags 						 ;
	mov ecx,[AT114NumberOfGroups]						 ;
	rep movsb								 ; */

	pop edi
	pop esi

; copy location flags, set up location pointers, and copy potential x,y,z values
; while freeing temporary memory. Also, convert x,y,z values (if present) to quaternions.
; Implicitly keeps order.

     ; esi = storage for x,y,z values
     ; ebx = counter for location values

	mov [AT114TempBufferBaseFlags],ebx
	add ebx,[AT114NumberOfGroupsXInstances]
	mov [AT114TempBufferBasePointers],ebx

	xor ebx,ebx

	.copygroupdataloop:

; copy flags

	mov edx,[AT114TempBufferBaseFlags]
	mov ecx,[edi+CAT114.lpGroupLocationFlags]
	mov al,[ebx+edx]
	mov [ebx+ecx],al

	test al,AT114_LOC_ENTRYVALID
	jz .leavyentry
	test al,AT114_LOC_USEDESTOBJECT
	jz .usexyz

; if dest. object, simply copy instance ID

	mov edx,[AT114TempBufferBasePointers]
	mov ecx,[edi+CAT114.lpGroupData]
	mov eax,[ebx*4+edx]
	mov [ebx*4+ecx],eax
	jmp .leavyentry

	.usexyz:

; if XYZ, check entry for orientation/position and copy accordingly

	mov DWORD [ebp-4],3
	and al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	cmp al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	mov BYTE [ebp-8],al
	jnz @f								    ; use both? if not, skip

		mov DWORD [ebp-4],6

	@@:

	mov edx,[AT114TempBufferBasePointers]
	mov ecx,[edi+CAT114.lpGroupData]
	mov eax,[ebx*4+edx]							; get x,y,z values in separate memory location
	mov [ebp-12],eax
	mov [ebx*4+ecx],esi							; store pointer in AT114 instance to new space for x,y,z values

	push edi
	mov edi,esi
	mov esi,eax
	mov ecx,[ebp-4]
	rep movsd								; copy all x,y,z values for this location
	mov edx,esi
	mov esi,edi
	pop edi

      ; if orientation values were used, convert the three last members
      ; (three angles) to a four-member quaternion (increase size by 4)

	test BYTE [ebp-8],AT114_LOC_USESETORIENTATION
	     jz .leaveentryandfree

	add esi,4
	lea ecx,[edx-12]
	lea edx,[esi-16]
	fld DWORD [ecx]
	fmul [pidiv180]
	fld DWORD [ecx+4]
	fmul [pidiv180]
	fld DWORD [ecx+8]
	fmul [pidiv180]
	fstp DWORD [ecx+8]
	fstp DWORD [ecx+4]
	fstp DWORD [ecx]
	call XYZDegreesToQuaternion

	.leaveentryandfree:

	invoke LocalFree,[ebp-12]

	.leavyentry:

	inc ebx
	cmp ebx,[AT114NumberOfGroupsXInstances]
	jb .copygroupdataloop

; finalize process

	invoke LocalFree,[AT114TempBufferBaseFlags]
	call RANDOMIZE_RANDOM_PROC	;seed random value in advance

	mov [AT114LastError],AT114_ERRORSTR_SUCCESS
	mov [AT114LastErrorDParam1],edi
	mov eax,[AT114InstanceSize]
	mov ecx,[AT114NumberOfGroups]
	mov [AT114LastErrorDParam2],eax
	mov [AT114LastErrorDParam3],ecx
	mov eax,[AT114NumberOfInstances]
	mov ecx,[AT114TotalProbabilityCount]
	mov [AT114LastErrorDParam4],eax
	mov [AT114LastErrorDParam5],ecx

	jmp .exitinitdone

.exitinit:

	mov eax,[AT114NumberOfGroupsXInstances]
	mov esi,ebx
	mov [ebp-12],ebx
	push edi
	lea edi,[esi+eax]
	add ebx,[AT114TempBufferSize]

	.tempbufferfreeloop:

	mov al,[esi]

	test al,AT114_LOC_ENTRYVALID
	jz @f
	test al,AT114_LOC_USEDESTOBJECT
	jnz @f

		invoke LocalFree,[edi]

	@@:

	add edi,4
	add esi,1
	cmp edi,ebx
	jb .tempbufferfreeloop

	invoke LocalFree,[ebp-12]
	pop edi

.noinitialvalues:


.exitinitdone:

; print last error message to ATX_LOGFILE.txt

	mov ebx,edi
	mov esi,AT114_ERRORSTR_MAIN
	mov edi,AT114MessageBuffer
	call TransferString

	mov [AT114TempVar],esp
	invoke sprintf,edi,[AT114LastError],[AT114LastErrorDParam1],\
		      [AT114LastErrorDParam2],[AT114LastErrorDParam3],\
		      [AT114LastErrorDParam4],[AT114LastErrorDParam5]
	add eax,AT114_ERRORSTR_MAIN.size
	mov esp,[AT114TempVar]

	stdcall WriteLogFileInGame,AT114MessageBuffer,eax
	mov edi,ebx

	cmp [AT114LastError],AT114_ERRORSTR_SUCCESS
	jnz UnableToAllocateMemNoMsgPrint

	jmp ExitINAT
