;============================================================
;MAIN NEWACTIONTYPE INITIALIZATION CODE
;============================================================

;====================================================================
ATXPROC INAT						; eax = AT number
;====================================================================
; -> Must return new object in esi if init succeeds
; DO NOT CHANGE STACK FRAME STRUCTURE OR REGISTER PUSH/POP ORDERING

push ebp
mov ebp,esp
sub esp,16						; possible local values, not necessarily used
push ecx
push edx
push ebx						; [ebp-28] - pointer to base of ID table ()
push esi						; [ebp-32] - pointer to CObjectValue instance (dynamic)
push edi						; [ebp-36] - pointer to CValueTable instance (local)

; check new ActionType number and pass on to proper handler

	mov edi,eax
	mov [CurrentATNumber],eax

	mov [NoNewATLastAbortReason],NEWAT_INITABORTMSG_BADATNUMBER

	sub eax,100
	cmp eax,NUM_NEWACTIONTYPES
	jnb CheckForNewPluginATs

	mov [NoNewATLastAbortReason],NEWAT_INITABORTMSG_DEFAULT

	xor edx,edx
	mov [ebp-16],edx
	mov [ebp-12],edx
	mov [ebp-8],edx
	mov [ebp-4],edx

	xor edi,edi					; new in atx 1.62
	jmp DWORD [ATINATTable+4*eax]			; jump to new AT initialization


CheckForNewPluginATs:

	cmp edi,PLUGIN_NEWAT_BASE_NUM
	jb NoNewActionType

	mov [NoNewATLastAbortReason],NEWAT_INITABORTMSG_PLUGDEFAULT

	mov eax,[ebp-28]
	mov edx,[ebp-36]
	mov [LastScriptIDTableAddr],eax
	mov [LastCValueTableInst],edx

	PLUGINATINITCBINVOKE edi,DWORD [ebp-32] 	; MAIN PLUGIN CALLBACK INVOKE

	test eax,eax
	jz NoNewActionTypeNoMsgPrint

	lea edi,[eax-sizeof.PLUGIN_NEWAT_HEADER]	; MUST SUBSTRACT HEADER SIZE FROM eax
	jmp ExitINAT


;====================================================================
; AT INIT JUMP TABLE (DWORD LIST)
;====================================================================

ATINATTable:

	rept 100 counter:100 {

		if defined _AT#counter#Init
		  dd _AT#counter#Init
		end if

	}

ATINATTable_End:

align 16
;====================================================================
; initialization success
;====================================================================

ExitINAT:

	mov eax,[AT_FOUND_JUMPDEST]

	mov esi,edi					; save new ActionType object's address in esi
	pop edi
	add esp,4
	pop ebx
	pop edx
	pop ecx
	mov esp,ebp
	pop ebp

	mov ebp,[esp+18h]				; seems standard amongst ActionTypes
	jmp eax

align 16
;====================================================================
; initialization failure - passive exit (permits continuing level load)
;====================================================================

NoNewActionTypeFreeMem: 				; here, memory to free in edi before exiting

	ccall [free],edi

NoNewActionType:					; exit silently, standard (game interprets as "actiontype number not recognized")

	call NewATWriteAbortMsgToLogFile

NoNewActionTypeNoMsgPrint:				; no message printing

	mov eax,[NO_AT_FOUND_JUMPDEST]

	pop edi
	pop esi
	pop ebx
	pop edx
	pop ecx
	mov esp,ebp
	pop ebp

	jmp eax

align 16
;====================================================================
; initialization failure - provoke loading level halt (crash)
;====================================================================

UnableToAllocateMem:

	mov [NoNewATLastAbortReason],NEWAT_INITABORTMSG_MEMORY
	call NewATWriteAbortMsgToLogFile

UnableToAllocateMemNoMsgPrint:

	mov eax,[AT_MEM_ERROR_JUMPDEST]

	pop edi
	pop esi
	pop ebx
	pop edx
	pop ecx
	mov esp,ebp
	pop ebp

	xor esi,esi					; new in ATX 1.62 (should always have been)

	mov ebp,[esp+18h]				; seems standard amongst ActionTypes
	jmp eax

