;=====================================================
ATXPROC SHOWMAP_CODE
;=====================================================
; DISPLAYS MAP ON SCREEN (BACK BUFFER)

; * does not perform any error-checking as it would have no use
; [ebp-4] = loaded blinker bitmap handle
; [ebp-8] = compatible DC (as temp buffer) of DX surface
; [ebp-12] = Y value of map blinker location
; [ebp-16] = X value of map blinker location
; edi = DX surface Device Context
; NOTE: ALL FUNCTIONS DRAW ON THE CURRENT DIRECTDRAW BACKBUFFER

push ebp
mov ebp,esp
sub esp,16

; get main DirectX surface from EXE

	GETDDSURFACEDC

; check if map exists

	cmp [MAPCurrentBitmap],0
	    jnz @f

	invoke TextOut,eax,0,0,MAP_NO_CURRENT_BMP_STR,MAP_NO_CURRENT_BMP_STR.size
	jmp .exitrelease

	@@:

; create alternate DC for blinker blitting

	invoke LoadBitmap,[dllhandle],100
	mov [ebp-4],eax

	invoke CreateCompatibleDC,edi				; DDsurface DC
	mov [ebp-8],eax

	invoke SelectObject,eax,[ebp-4]

; draws bitmap directly to screen (no real need for double-buffering)

	mov ecx,[MAPCurrentBitmap]
	add ecx,14						; BITMAPINFO start
	mov eax,[ecx-4]
	add eax,[MAPCurrentBitmap]				; DIB bits start

	invoke SetDIBitsToDevice,edi,6,6,[MAPBitmapWidth],\
	       [MAPBitmapHeight],0,0,0,[MAPBitmapHeight],eax,ecx,DIB_RGB_COLORS

; draw dino positions on map (if enabled)

	cmp [ShowDinosOnMap],0
	    jz @f

	call DrawCAnimalInstancesOnMap

	@@:

; draw player path traces on map

	test [PTDisplayed],1
	     jz @f

	call DrawPlayerPathTraces

	@@:

; draw player 'blinker' to screen (over map)

	test [ShowPlayerBlinker],1
	     jz @f

	call DrawPlayerBlinker

	@@:

; calculate whether to display blinker next

	invoke GetTickCount
	mov edx,eax

	mov ecx,[MAPLastTickCount]
	sub eax,ecx
	cmp eax,PLAYER_BLINKER_INTERVAL
	    jb @f

	mov [MAPLastTickCount],edx
	xor BYTE [ShowPlayerBlinker],1

	@@:

; exit code

	invoke DeleteDC, DWORD [ebp-8] ;delete compatible DC

	invoke DeleteObject,DWORD [ebp-4]

.exitrelease:

	RELEASEDDSURFACEDC

mov esp,ebp
pop ebp
ret

;=====================================================
ATXPROC DrawCAnimalInstancesOnMap
;=====================================================

	mov esi,[CAnimalBuffer]
	test esi,esi
	     jz .exit

	.showloop:
	push DWORD CANIMAL_CURSOR_COLOR
	mov ebx,[esi+CANIMAL_INFO.Instance]

	call ShowCurrentAnimalOnMap_RT

	add esi,sizeof.CANIMAL_INFO
	cmp esi,[CAnimalBufferEnd]
	    jb .showloop

	cmp [ShowActiveDinoOnMap],0
	    jz .exit

	mov ecx,[CAnimalDisplayIndex]
	mov eax,[CAnimalBuffer]
	mov ebx,[eax+ecx+CANIMAL_INFO.Instance] 		; ebx=selected CAnimal instance
	push CANIMAL_CURSOR_SELECTED_COLOR
	call ShowCurrentAnimalOnMap_RT

.exit:
ret

;=====================================================
ATXPROC ShowCurrentAnimalOnMap_RT	 ; ebx=CAnimal object, [esp+4]=color
;=====================================================

	fld DWORD [ebx+30h]
	fsub [MAPAnchorXValue]					; substract top-corner X-anchor from Xposition
	fdiv [MAPScaleXValue]					; Xfloatdisplacement / (X float units/pixel) = # pixels (Xpos)
	fistp DWORD [ebp-16]					; convert to integer and store

	fld DWORD [ebx+34h]
	fsub [MAPAnchorYValue]					; substract top-corner Y-anchor from Xposition
	fdiv [MAPScaleYValue]					; Yfloatdisplacement / (Y float units/pixel) = # pixels (Ypos)
	fistp DWORD [ebp-12]					; convert to integer and store

	xor edx,edx

	mov eax,[ebp-16]					; outside map (to the left)?
	test eax,eax
	     jns @f
	or edx,1
	@@:

	cmp eax,[MAPBitmapWidth]					; outside map (to the right)?
	    jb @f
	or edx,1
	@@:

	add eax,(6-2)						; map displacement - cursor size
	mov [ebp-16],eax

	mov eax,[ebp-12]					; outside map (upward)?
	test eax,eax
	     jns @f
	or edx,1
	@@:

	cmp eax,[MAPBitmapHeight]					; outside map (downward)?
	    jb @f
	or edx,1
	@@:

	add eax,(6-2)						; map displacement - cursor size
	mov [ebp-12],eax

	test edx,edx
	     jnz .exit

; DRAW CURSOR

	invoke SetPixelV,edi,[ebp-16],[ebp-12],[esp+4]

	mov eax,[ebp-16]
	dec eax
	invoke SetPixelV,edi,eax,[ebp-12],[esp+4]

	mov eax,[ebp-16]
	sub eax,2
	invoke SetPixelV,edi,eax,[ebp-12],[esp+4]

	mov eax,[ebp-16]
	inc eax
	invoke SetPixelV,edi,eax,[ebp-12],[esp+4]

	mov eax,[ebp-16]
	add eax,2
	invoke SetPixelV,edi,eax,[ebp-12],[esp+4]


	mov edx,[ebp-12]
	dec edx
	invoke SetPixelV,edi,[ebp-16],edx,[esp+4]

	mov edx,[ebp-12]
	sub edx,2
	invoke SetPixelV,edi,[ebp-16],edx,[esp+4]

	mov edx,[ebp-12]
	inc edx
	invoke SetPixelV,edi,[ebp-16],edx,[esp+4]

	mov edx,[ebp-12]
	add edx,2
	invoke SetPixelV,edi,[ebp-16],edx,[esp+4]

.exit:
ret 4

;=====================================================
ATXPROC DrawPlayerBlinker
;=====================================================
; ebx = used to test if position is in map

; Calculate blinker position in map

	GETPLAYERLOCADDR					; eax points to x,y,z positions

	fld DWORD [eax]
	fsub [MAPAnchorXValue]					; substract top-corner X-anchor from Xposition
	fdiv [MAPScaleXValue]					; Xfloatdisplacement / (X float units/pixel) = # pixels (Xpos)
	fistp DWORD [ebp-16]					; convert to integer and store

	fld DWORD [eax+4]
	fsub [MAPAnchorYValue]					; substract top-corner Y-anchor from Xposition
	fdiv [MAPScaleYValue]					; Yfloatdisplacement / (Y float units/pixel) = # pixels (Ypos)
	fistp DWORD [ebp-12]					; convert to integer and store

; test position and clip blinker to known map edges

	mov ebx,SRCPAINT

	mov eax,[ebp-16]					; outside map (to the left)?
	test eax,eax
	     jns @f
	xor eax,eax
	mov ebx,SRCAND
	@@:

	cmp eax,[MAPBitmapWidth]					; outside map (to the right)?
	    jb @f
	mov eax,[MAPBitmapWidth]
	dec eax
	mov ebx,SRCAND
	@@:

	;sub eax,6                                               ; convert location point from center of blinker to corner
								 ; EDIT: no longer needed -> map displaced instead
	mov [ebp-16],eax

	mov eax,[ebp-12]					; outside map (upward)?
	test eax,eax
	     jns @f
	xor eax,eax
	mov ebx,SRCAND
	@@:

	cmp eax,[MAPBitmapHeight]					;outside map (downward)?
	    jb @f
	mov eax,[MAPBitmapHeight]
	dec eax
	mov ebx,SRCAND
	@@:

	;sub eax,6

	mov [ebp-12],eax

	invoke BitBlt,edi,DWORD [ebp-16],DWORD [ebp-12],13,13,DWORD [ebp-8],0,0,ebx

ret