;============================================================
ATXPROC LOADINITIALBITMAP
;============================================================
; Checks current level file and compares to entries in INI file.
; Called from POSTLEVELLOADINTERCEPT (first call).
; -> see 'saving' for short description of SAVEDMAP.xxx file format
; [ebp-4] = file handle
; [ebp-8] = file mapping handle (if used/not saved game)
; ebx = EOF marker

push ebp
mov ebp,esp
sub esp,8
push esi
push edi
push ebx

	xor eax,eax
	mov [ebp-4],eax
	mov [ebp-8],eax

	mov esi,[LEVELNAME_OBJ] 				; (esi points to level SCN name)
	add esi,20						; (esi points to savegame/scn file name and path)
	mov edi,SaveGameNameBuffer				; PATH+NAME OF CURRENT LEVEL FILE COPY (OR SAVEGAME)
	call TransferString

	call IsLevelSavedGame
	test eax,eax
	     jz .loadasscn


; assume it is otherwise a saved game (LOAD SAVEDMAP.xxx AND ALL COMPONENTS)
; note: some savegame-loaded settings are pre-initialized in PRELEVELLOAD.inc in
; the case where there is no savegame.

	mov DWORD [edi-8],'DMAP'				; possible save extension saved as 'SAVEDMAP.xxx'

	invoke CreateFile,SaveGameNameBuffer,GENERIC_READ,0,0,\
			  OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL+FILE_FLAG_SEQUENTIAL_SCAN,0
	test eax,eax
	     jz .loadfail

	cmp eax,-1
	    jz .loadfail
	mov [ebp-4],eax

	invoke GetFileSize,eax,0
	cmp eax,-1
	    jz .loadfail

	cmp eax,21						; at least 4 32-bit floats + 'x.bmp'
	    jb .loadfail

	invoke SetFilePointer,DWORD [ebp-4],0,0,FILE_BEGIN
	cmp eax,-1
	    jz .loadfail

; read 4 POSITION float values and store

	SGLOADREAD SavedMapX1,16,.loadfail

; read BITMAP name and store

	lea eax,[ebp-8]
	invoke ReadFile,DWORD [ebp-4],SavedMapBuffer,80,eax,0
	test eax,eax
	     jz .loadfail

	cmp DWORD [ebp-8],0
	    jna .loadfail

	cmp DWORD [ebp-8],80
	    jnz .loadabortatgspd

; Read saved game speed

	SGLOADREAD stack_ext_buffer,4,.loadabortatgspd

	cmp [OverrideSVGSpeed],FALSE
	    jnz @f
	mov eax,DWORD [stack_ext_buffer]
	mov edx,[GAME_SPEED_VAR_ADDR]
	mov [NormalGameSpeed],eax
	mov [edx],eax						; set initial game speed
	@@:

; Read AT106 Hint String and set initial hint

	SGLOADREAD stack_ext_buffer,4,.loadabortatgspd		; get string length

	mov edi,DWORD [stack_ext_buffer]
	xor ebx,ebx
	test edi,edi
	     jz .noat106hintload

	invoke LocalAlloc,LMEM_FIXED,edi			; allocate mem for string
	test eax,eax
	mov ebx,eax
	     jz .loadabortatgspd

	lea eax,[ebp-8]
	invoke ReadFile,DWORD [ebp-4],ebx,edi,eax,0		; get string
	test eax,eax
	     jz .at106hintloadfail

	cmp DWORD [ebp-8],edi
	    jz .at106loadhintok

	.at106hintloadfail:
	invoke LocalFree,ebx
	jmp .loadabortatgspd

	.at106loadhintok:
	LOCALRELEASE [HintNewMessageAddr]
	mov [HintNewMessageAddr],ebx
	mov eax,[CURRENT_HINT_ID_ADDR]
	mov [eax],DWORD NEW_HINT_TYPE_ID			; set new initial Hint string for level

	.noat106hintload:

; Read AT110 BUFFER

	SGLOADREAD stack_ext_buffer,4,.loadabortatgspd		; get buffer length

	mov edi,DWORD [stack_ext_buffer]
	xor ebx,ebx
	test edi,edi
	     jz .at110loadok

	cmp edi,16
	    jb .loadabortatgspd

	invoke LocalAlloc,LMEM_FIXED,edi
	test eax,eax
	mov ebx,eax
	     jz .loadabortatgspd

	lea eax,[ebp-8]
	invoke ReadFile,DWORD [ebp-4],ebx,edi,eax,0		; fill buffer
	test eax,eax
	     jz .at110loadfail

	cmp DWORD [ebp-8],edi
	    jz .at110loadok

	.at110loadfail:
	invoke LocalFree,ebx
	jmp .loadabortatgspd

	.at110loadok:
	mov [FixSSaveDataTempBuf],ebx
	mov [FixSSaveDataTempBUFSIZE],edi

; Read AT112 AND AT113 VALUES

	SGLOADREAD stack_ext_buffer,12,.loadabortatgspd 	; get buffer length

	mov eax,DWORD [stack_ext_buffer]
	mov ecx,DWORD [stack_ext_buffer+4]
	mov edx,DWORD [stack_ext_buffer+8]

	mov [AT113SaveGameAITypeUsed],eax
	mov [AT113SaveGameAITypeValue],ecx
	mov [at112inputtogglerid],edx

; Read AT118 VALUES (called BEFORE normal buffer alloc - AT118InitInstRefBuffer)

   ; read header values (size of following buffer)

	SGLOADREAD stack_ext_buffer,sizeof.AT118_SGBLOCKHEADER,.loadabortatgspd

	cmp [stack_ext_buffer+AT118_SGBLOCKHEADER.InstanceCount],0
	mov esi,[stack_ext_buffer+AT118_SGBLOCKHEADER.InstRefBufCut]
	jz .noAT118readsgbuffer

	test esi,esi
	jz .noAT118readsgbuffer

   ; create memory buffer

	mov eax,[stack_ext_buffer+AT118_SGBLOCKHEADER.InstanceCount]
	cmp eax,[AT118BufDesc.InstanceCount]
	jbe @f						    ; use max of instance counts (in case level changes have occured)
		mov [AT118BufDesc.InstanceCount],eax
	@@:
	mov [AT118BufDesc.InstRefBufCut],0

	call AT118InitInstRefBuffer				; allocate buffer in advance

   ; read following buffer into memory

	cmp [AT118InstanceRefBuf],0
	jz .noAT118readsgbuffer

	SGLOADREAD [AT118InstanceRefBuf],esi,.loadabortatgspd
	mov [AT118BufDesc.InstRefBufCut],esi
	call AT118ReapplyAllRecordedTextureChanges		; re-apply all textures changes from savegame

	.noAT118readsgbuffer:

; Read Rain Effect/AT119 values

	SGLOADREAD CurrentRainEnabled,4,.loadabortatgspd
	SGLOADREAD stack_ext_buffer,sizeof.RainEffectBase,.loadabortatgspd

	cmp [stack_ext_buffer+RainEffectBase.EffectNameID],0
	jz .nocustomraineffect

		push esi
		push edi
		mov esi,stack_ext_buffer
		mov edi,CurrentRainEffect
		mov ecx,(sizeof.RainEffectBase)/4
		rep movsd
		pop edi
		pop esi

		mov ecx,CurrentRainEffect
		call RainEffectCalcGridSizes

	.nocustomraineffect:

.loadabortatgspd:

; LOAD BITMAP FROM SAVEDMAP.xxx

	mov ecx,SavedMapObject
	call AT102MAIN
	test eax,eax						; if load fails, use level's originally indexed map
	     jnz .exitload					; (jumps if success)

.loadfail:

.loadasscn:

; scan for bitmap name in/from level name section; if found, scan for X/Y position values

	cmp [MapLoadSuccess],TRUE				; Has a map already been successfully loaded via CStartTrigger?
	    jz .exitload

	mov edi,mapsdirpath
	mov esi,MAPINDEX_FILENAME				; INI filename
	add edi,[mapsdirpathlength]
	call TransferString

	mov esi,[LEVELNAME_OBJ] 				; addr of SCN level name string (section)

	mov DWORD [AT102EMULATEDOBJECT+8],BitmapMapNameBuffer
	invoke GetPrivateProfileString,esi,STRING_BITMAP,STRING_BMPDEF,BitmapMapNameBuffer,79,mapsdirpath
	cmp DWORD [BitmapMapNameBuffer],'DEF'
	    jz .newbmploadfail

	invoke GetPrivateProfileFloat,esi,STRING_X1,ZERO_STRING,mapsdirpath
	mov [AT102EMULATEDOBJECT+12],eax
	invoke GetPrivateProfileFloat,esi,STRING_Y1,ZERO_STRING,mapsdirpath
	mov [AT102EMULATEDOBJECT+16],eax
	invoke GetPrivateProfileFloat,esi,STRING_X2,ZERO_STRING,mapsdirpath
	mov [AT102EMULATEDOBJECT+20],eax
	invoke GetPrivateProfileFloat,esi,STRING_Y2,ZERO_STRING,mapsdirpath
	mov [AT102EMULATEDOBJECT+24],eax

; Load new bitmap (if not saved game)

	mov ecx,AT102EMULATEDOBJECT
	call AT102MAIN						; emulates a normal AT102 call to load file
	test eax,eax
	     jnz .exitload					; (jump if success)


.newbmploadfail:

	LOCALRELEASE [MAPCurrentBitmap] 			; unload old map if existing

.exitload:

	CLOSEHANDLE DWORD [ebp-4]

pop ebx
pop edi
pop esi
mov esp,ebp
pop ebp
ret
