;=====================================================
; create map index save file for savegame
;=====================================================

; [DWORD [CPARTICLESBUFADDRVAR] - 16] = pointer to list of
; particle name objects (size = 6 DWORDS) - Type ID is member +16
; [DWORD [CPARTICLESBUFADDRVAR] + 4] = number of CParticle objects

; SAVEDMAP.xxx format (starting at offset 0):
;
; 16 bytes       - 2D Map X-Y position values (4 32-bit floats)
; 80 bytes       - 2D Map BMP name (*.bmp+null-terminator)
; 4 bytes        - Game speed value (32-bit float)
; 4 bytes        - Number of bytes in "current hint" string (0 if none)
; variable       - "current hint" string

; note: every savegame 'block' always begins with a certain value
; of fixed size, such as a DWORD indicating the size of the rest of the
; block to follow. EVERY BLOCK MUST HAVE SUCH A KIND OF FIXED-SIZE HEADER, i.e.,
; it cannot simply 'not exist' in the savegame (the loading code
; depends on all blocks existing because they are loaded simply based
; on the order in which they are found in the file).


;=====================================================
INTERCEPTPROC SAVE_SAVED_MAP_STATE			; ebx = file handle
;=====================================================

mov edx,[esp+1E4h]					; REPLACED CODE (7 bytes)
							; edx = pointer to SAVEGAME.xxx filename + path
push ebx
push ecx
push esi
push edi

	xor ebx,ebx

	mov esi,edx
	mov edi,SaveGameNameBuffer
	call TransferString

	cmp DWORD [edi-4],'.scx'
	jz .exitsavemap

	cmp DWORD [edi-4],'.SCX'
	jz .exitsavemap

	mov DWORD [edi-8],'DMAP'			; 'SAVEGAME' replaced by 'SAVEDMAP'



	cmp [MAPCurrentBitmap],0			; if no map, make sure to write only null characters
	jnz @f

		push edi
		xor eax,eax
		mov edi,CurrentMapValues
		mov ecx,4
		rep stosd
		mov edi,CurrentMapBuffer
		mov ecx,20
		rep stosd
		pop edi

	@@:

	invoke SetFileAttributes,SaveGameNameBuffer,FILE_ATTRIBUTE_NORMAL	; make sure "SAVEDMAP.xxx" writable!    ; new in ATX 2.06

	invoke CreateFile,SaveGameNameBuffer,GENERIC_WRITE,0,0,\
			  CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL+FILE_FLAG_SEQUENTIAL_SCAN,0
	test eax,eax
	jz .exitsavemap

	cmp eax,-1
	jz .exitsavemap

	mov ebx,eax

	invoke SetFilePointer,eax,0,0,FILE_BEGIN
	cmp eax,-1
	jz .exitsavemap

; SAVE MAP POSITION VALUES AND 2D MAP NAME

	SGSAVEWRITE CurrentMapValues,16,.exitsavemap			; write POSITION values
	SGSAVEWRITE CurrentMapBuffer,80,.exitsavemap			; write BITMAP NAME

; SAVE CURRENT GAME SPEED

	mov eax,[NormalGameSpeed]
	mov DWORD [stack_ext_buffer],eax

	SGSAVEWRITE stack_ext_buffer,4,.exitsavemap			; write GAME SPEED

; SAVE CURRENT HINT (IF AT106-TYPE)

	mov edx,[CURRENT_HINT_ID_ADDR]
	mov eax,[HintNewMessageAddr]
	cmp DWORD [edx],NEW_HINT_TYPE_ID
	jnz .nosavecurrenthint

	test eax,eax
	jz .nosavecurrenthint

	mov edi,eax
	FINDSTRINGEND edi
	sub edi,eax
	inc edi

	mov DWORD [stack_ext_buffer],edi
	SGSAVEWRITE stack_ext_buffer,4,.exitsavemap			; write HINT string length
	SGSAVEWRITE [HintNewMessageAddr],edi,.exitsavemap		; write HINT strings

	jmp .hintsaved

	.nosavecurrenthint:
	mov DWORD [stack_ext_buffer],0
	SGSAVEWRITE stack_ext_buffer,4,.exitsavemap			; write HINT string length as NULL

	.hintsaved:

; BUILD AND SAVE AT110 DATA BUFFER

	push esi

	mov esi,[FixSMainBuffer]
	xor ecx,ecx
	test esi,esi
	mov edi,[FixSMainBufferSizeDiv2]
	jz .noat110events

	xor eax,eax

	.at110countloop:
	cmp [esi],eax
	jz @f
		inc ecx
	@@:
	add esi,4
	cmp esi,[FixSMainBufferHalfEnd]
	jb .at110countloop

	test ecx,ecx
	jz .noat110events

	mov eax,(4+sizeof.AT110_PROJ_INFO)
	mul ecx
	lea ecx,[eax+8] 						; add space for struc size and AT110 version ID

	.noat110events: 						; ...but must still write null size count

	pop esi

	mov DWORD [stack_ext_buffer],ecx
	SGSAVEWRITE stack_ext_buffer,4,.exitsavemap

	mov ecx,DWORD [stack_ext_buffer]
	test ecx,ecx
	jz .exitat110save

	invoke LocalAlloc,LMEM_FIXED,ecx
	test eax,eax
	mov edi,eax
	jz .exitsavemap

    ; generate temporary buffer to write

	push ebx
	push ebp
	push esi
	push edi

	mov DWORD [edi],AT110_VERSION_ID
	mov DWORD [edi+4],sizeof.AT110_PROJ_INFO
	add edi,8

    ; BUFFER FORMAT: AT110 Version ID, AT110_PROJ_INFO structure size,
    ;                instance name ID, structure, instance name ID, structure, etc.

	mov esi,[FixSMainBuffer]
	mov ebx,[FixSMainBufferSizeDiv2]

	.saveat110loop:
	mov eax,[esi]
	test eax,eax
	jz @f

		mov [edi],eax						   ; copy name ID
		add edi,4
		push esi
		mov esi,[esi+ebx]
		mov ecx,(sizeof.AT110_PROJ_INFO)
		rep movsb						   ; copy structure
		pop esi

	@@:
	add esi,4
	cmp esi,[FixSMainBufferHalfEnd]
	jb .saveat110loop

	pop edi
	pop esi
	pop ebp
	pop ebx

	sub esp,4
	mov eax,esp
	invoke WriteFile,ebx,edi,DWORD [stack_ext_buffer],eax,0 	; write final buffer

	push eax
	invoke LocalFree,edi
	pop eax
	pop ecx
	test eax,eax
	jz .exitsavemap

	cmp ecx,DWORD [stack_ext_buffer]
	jnz .exitsavemap

.exitat110save:

; SAVE AT112- AND AT113-SET VALUES

	xor ecx,ecx
	mov cl,[AT113PlayerAITypeNon5Set]
	mov eax,[at112inputtogglerid]
	mov DWORD [stack_ext_buffer],ecx
	mov DWORD [stack_ext_buffer+8],eax

	call GetPlayerPrivAIType
	mov DWORD [stack_ext_buffer+4],eax

	SGSAVEWRITE stack_ext_buffer,12,.exitsavemap

; SAVE AT118 BUFFER (if any)

	SGSAVEWRITE AT118BufDesc,sizeof.AT118_SGBLOCKHEADER,.exitsavemap	; write essential header

	mov edx,[AT118InstanceRefBuf]
	test edx,edx
	jz @f

		SGSAVEWRITE edx,[AT118BufDesc.InstRefBufCut],.exitsavemap		; write buffer contents if buffer exists - ONLY USED ENTRIES ARE OUTPUTTED

	@@:

; SAVE RAIN EFFECT STATE

	SGSAVEWRITE CurrentRainEnabled,4,.exitsavemap
	SGSAVEWRITE CurrentRainEffect,sizeof.RainEffectBase,.exitsavemap


.exitsavemap:

	CLOSEHANDLE ebx


	mov edx,[esp+1E4h+16]
	PLUGINSCBINVOKE PLUGIN_CALLBACKS.OnLevelSave,edx

pop edi
pop esi
pop ecx
pop ebx

mov edx,[esp+1E4h]							; restore edx
ret

