;============================================================
ATXPROC DoAnimalProjectileInit
;============================================================
push ebx
push ebp
push esi
push edi

	call ReleaseAnimalProjBuffer

	cmp [CAnimalBuffer],0
	    jz .exit

	call GetCAnimalInstanceCount
	test eax,eax
	     jz .exit

	shl eax,2								; get size of buffer
	mov ebp,eax
	invoke LocalAlloc,LPTR,eax
	test eax,eax
	     jz .exit

	mov [AnimalProjectileBuffer],eax
	mov [AnimalProjectileBufferSize],ebp

	mov esi,[LEVELNAME_OBJ] 						; SCN level name
	mov edi,mapsdirpath
	add edi,[mapsdirpathlength]
	mov DWORD [edi],SPITTER_MAPS_FOLDER					; 'SPT\' folder for jump data
	add edi,4
	call TransferString
	mov DWORD [edi-4],EXTENSION_INI_UPPERCASE				; modify extension

	mov ebx,[CAnimalBuffer]
	mov esi,[AnimalProjectileBuffer]

	push DWORD 0								; will be set to 1 if at least one spitter

; MAIN LOOP - scan for possible values in sections named after animals (1 iteration per animal/section)

	.mainloop:
	mov eax,[ebx+CANIMAL_INFO.Instance]
	mov edx,[eax+0DCh]							; CBrain
	lea ebp,[edx+220h]							; ebp=pointer to instance/section name

	; scan for projectile names loop (1 - 8)
	    mov edi,1
	    .projectilenamesloop:

	    cinvoke wsprintf,stack_ext_buffer,STRING_PROJECTILENAME_COUNT,edi

	    invoke GetPrivateProfileString,ebp,stack_ext_buffer,ZERO_STRING,(stack_ext_buffer+200),200,mapsdirpath
	    sub esp,4
	    cmp WORD [(stack_ext_buffer+200)],'0'				; includes null-terminator
		jz .noprojectile
		mov eax,(stack_ext_buffer+200)
		call GetInstanceFromStringGlobal
		test eax,eax
		mov [esp],eax
		     jz .noprojectile
		     mov edx,[esi]						; if no buffer, create one
		     test edx,edx
			  jnz .nonewbuf
			  invoke LocalAlloc,LPTR,sizeof.ANIM_PROJ_INFO
			  test eax,eax
			  mov edx,eax
			      jz .noprojectile
			  mov [esi],edx
			  lea eax,[edi*4-4-4]					; projectile index (will point to -4 for first instance)
			  mov DWORD [esp+4],00000001				; at least one spitter
			  mov [edx+ANIM_PROJ_INFO.NextProjectileIndex],eax
		     .nonewbuf:
		     mov eax,[esp]						; projectile instance
		     mov [edx+edi*4-4+ANIM_PROJ_INFO.ProjectileInstance],eax
		     inc [FixSSPTProjectileCount]
		.noprojectile:
		add esp,4

	    inc edi
	    cmp edi,8
		jbe .projectilenamesloop

	mov edi,[esi]
	test edi,edi								; only read other values if there is at least one projectile
	     jz .mainloopcontinue

    ; reset tick count
	mov [edi+ANIM_PROJ_INFO.LastTick],0

    ; set MinDistanceEXP2
	invoke GetPrivateProfileFloat,ebp,STRING_MINDISTANCE,ZERO_STRING,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.MinDistanceEXP2],eax
	fld [edi+ANIM_PROJ_INFO.MinDistanceEXP2]
	fmul st0,st0
	fstp [edi+ANIM_PROJ_INFO.MinDistanceEXP2]

    ; set MinDistanceHorizEXP2
	invoke GetPrivateProfileFloat,ebp,STRING_MINDISTANCEHORIZ,ZERO_STRING,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.MinDistanceHorizEXP2],eax
	fld [edi+ANIM_PROJ_INFO.MinDistanceHorizEXP2]
	fmul st0,st0
	fstp [edi+ANIM_PROJ_INFO.MinDistanceHorizEXP2]

    ; set ForwardDistanceFromHead
	invoke GetPrivateProfileFloat,ebp,STRING_FORWARDDISTANCEFROMHEAD,ZERO_STRING,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.DistanceFromHead],eax

    ; set ProjectileSpeed
	invoke GetPrivateProfileFloat,ebp,STRING_PROJECTILESPEED,ZERO_STRING,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.ProjectileSpeed],eax

    ; set LaunchProbability
	invoke GetPrivateProfileInt,ebp,STRING_LAUNCHPROBABILITY,100,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.LaunchProbability],eax

    ; set MinLaunchInterval
	invoke GetPrivateProfileInt,ebp,STRING_MINLAUNCHINTERVAL,3000,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.MinInterval],eax

    ; set ProbabilityInterval
	invoke GetPrivateProfileInt,ebp,STRING_PROBABILITYINTERVAL,-1,mapsdirpath
	cmp eax,-1
	    jnz @f
	    mov eax,[edi+ANIM_PROJ_INFO.MinInterval]
	@@:
	mov [edi+ANIM_PROJ_INFO.ProbabilityInterval],eax

    ; set ProjectileExtinguishTime
	invoke GetPrivateProfileInt,ebp,STRING_PROJECTILEEXTINGUISHTIME,-1,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.ProjectileExtinguishTime],eax

    ; set ProjectileZSpeedAdjust
	invoke GetPrivateProfileFloat,ebp,STRING_PROJECTILEZSPEEDADJUST,ZERO_STRING,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.ProjectileZSpeedAdjust],eax

    ; set UseHorizontalAutoAim
	invoke GetPrivateProfileInt,ebp,STRING_USEHORIZONTALAUTOAIM,FALSE,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.UseHorizontalAutoAim],al

    ; set UseVerticalAutoAim
	invoke GetPrivateProfileInt,ebp,STRING_USEVERTICALAUTOAIM,FALSE,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.UseVerticalAutoAim],al

    ; set AnimalViewAngle
	invoke GetPrivateProfileFloat,ebp,STRING_ANIMALVIEWANGLE,ZERO_STRING,mapsdirpath
	mov [edi+ANIM_PROJ_INFO.AnimalViewAngle],eax
	fld [edi+ANIM_PROJ_INFO.AnimalViewAngle]
	fmul [pidiv180] 							; convert to radians
	fst [edi+ANIM_PROJ_INFO.AnimalViewAngle]
	fcos
	fstp [edi+ANIM_PROJ_INFO.AnimalViewAngleCosine]

	.mainloopcontinue:
	add esi,4
	add ebx,sizeof.CANIMAL_INFO
	cmp ebx,[CAnimalBufferEnd]
	    jb .mainloop

	pop eax
	cmp eax,1								; release buffer if no spitters at all
	    jz @f
	    call ReleaseAnimalProjBuffer
	@@:

	call RANDOMIZE_RANDOM_PROC						; seed randomized value in advance

.exit:
pop edi
pop esi
pop ebp
pop ebx
ret

;============================================================
ATXPROC ReleaseAnimalProjBuffer
;============================================================
push esi
push ebx

	mov esi,[AnimalProjectileBuffer]
	mov ebx,[AnimalProjectileBufferSize]
	test esi,esi
	    jz .norelease2
	test ebx,ebx
	     jz .norelease
	add ebx,esi

	.releaseloop:
	mov eax,[esi]
	add esi,4
	test eax,eax
	     jz @f
	     invoke LocalFree,eax
	@@:
	cmp esi,ebx
	    jb .releaseloop

	.norelease:
	LOCALRELEASE [AnimalProjectileBuffer]

	.norelease2:
	xor eax,eax
	mov [AnimalProjectileBufferSize],eax
	mov [FixSSPTProjectileCount],eax

pop ebx
pop esi
ret

;============================================================
ATXPROC SpitterCode_RT
;============================================================
push ebp
mov ebp,esp
sub esp,(36+12)
push edi
push esi
push ebx

	cmp [CurrentOnActBiteTargetBiteInstance],0
	    jz .exit

	call GetCurrentOnActBiteAnimalInfo
	test eax,eax
	     jz .exit

	mov ecx,[eax+CANIMAL_INFO.Index]
	mov edx,[AnimalProjectileBuffer]
	mov esi,[eax+CANIMAL_INFO.Instance]

	mov ebx,[ecx*4+edx]
	test ebx,ebx
	     jz .exit							; is instance considered a spitter?

    ; ebx = ANIM_PROJ_INFO data struc for instance

;------------------------------------------
;CHECK TIME VALUES AND LAUNCH PROBABILITY
;------------------------------------------

    ; make sure enough time has elapsed (SINCE LAST PROJECTILE LAUNCH)

	invoke GetTickCount
	mov edi,eax							; save tick
	sub eax,[ebx+ANIM_PROJ_INFO.LastTick]
	cmp eax,[ebx+ANIM_PROJ_INFO.MinInterval]
	    jb .exit

    ; make sure enough time has elapsed (SINCE LAST PROBABILITY CHECK)

	mov eax,edi
	sub eax,[ebx+ANIM_PROJ_INFO.ProbabilityInterval_LastTick]
	cmp eax,[ebx+ANIM_PROJ_INFO.ProbabilityInterval]
	    jb .exit
	mov [ebx+ANIM_PROJ_INFO.ProbabilityInterval_LastTick],edi	; update launch probability tick

    ; random value check for launch probability

	stdcall GetRandomNumber,100
	cmp eax,[ebx+ANIM_PROJ_INFO.LaunchProbability]
	    ja .exit

;------------------------------------------
;BEGIN LAUNCH CODE
;------------------------------------------

    ; get biting target instance

	mov edi,[CurrentOnActBiteTargetBiteInstance]

    ; get dino's head boundary box instance
	mov esi,[esi+CAnimal.lpCBoundaryBoxHead]

    ; The following is not necessary (target can be of any type) /*
    ; ;get biting target instance (CInfluence pointer parameter's source instance)
    ;     mov edi,[CurrentOnActBiteTargetBiteInstance]
    ;     mov edx,[edi]
    ;     cmp edx,[CANIMALVTABLE]
    ;         jz SpitterCode_RT_BiteTargetFound
    ;     cmp edx,[CPLAYERPRIVVTABLE]
    ;         jz SpitterCode_RT_BiteTargetFound
    ;     cmp edx,[CBOUNDARYBOXVTABLE]                                   ; accepts only these base object types
    ;         jnz .exit
    ;         mov edi,[edi+CBoundaryBox.lpParentInstance]
    ;         mov edx,[edi]
    ;         cmp edx,[CANIMALVTABLE]
    ;             jz SpitterCode_RT_BiteTargetFound
    ;         cmp edx,[CPLAYERPRIVVTABLE]
    ;             jnz .exit                                              ; if CBoundaryBox (most likely), ensure parent is Anne or dino and fetch
    ;     SpitterCode_RT_BiteTargetFound:
    ;*/

    ; edi = target bite instance
    ; esi = biting CAnimal instance's head boundary box
    ; ebx = ANIM_PROJ_INFO struct for biting CAnimal

    ; make sure there is enough horizontal-only distance between the two instances

	fld [esi+CAnimal.Location.X]
	fsub [edi+CInstance.Location.X]
	fmul st0,st0
	fld [esi+CAnimal.Location.Y]
	fsub [edi+CInstance.Location.Y]
	fmul st0,st0
	faddp st1,st0
	fst DWORD [ebp-4]
	fcomp [ebx+ANIM_PROJ_INFO.MinDistanceHorizEXP2]
	fstsw ax
	test ah,1h							; (X-Y distance)^2 lower than (min distance horiz)^2?
	     jnz .exit

    ;make sure there is enough 3d distance between the two instances
	fld [esi+CAnimal.Location.Z]
	fsub [edi+CInstance.Location.Z]
	fmul st0,st0
	fadd DWORD [ebp-4]
	fst DWORD [ebp-8]
	fcomp [ebx+ANIM_PROJ_INFO.MinDistanceEXP2]
	fstsw ax
	test ah,1h							; (X-Y-Z distance)^2 lower than (min distance)^2?
	     jnz .exit

       ; edi = target bite instance
       ; esi = biting CAnimal instance's head boundary box
       ; ebx = ANIM_PROJ_INFO struct for biting CAnimal

    ; make sure dino's head is facing towards target horizontally with view angle check
	; a = angle to get, b = limit angle, vectors are X-Y only (horizontal only)
	;                   -> assumed 0 <= b <= pi
	; -> limit angle is for each side of head, not total view angle
	; cos(a) = vector1.vector2 / (sizeof.vector1*sizeof.vector2)     ;. = dot product ; both vectors assumed non-zero
	; validity equation:      a <= b  (for any angle between 0 and 90)
	;                    cos(a) >= cos(b)
	;                    vector1.vector2 / (sizeof.vector1*sizeof.vector2) >= cos(b)
	;                    vector1.vector2 >= cos(b)*(sizeof.vector1*sizeof.vector2)      (valid since vector sizes will never change sign, always positive)
	;                    vector1.vector2 >= cos(b)*sizeof.vector1*sizeof.vector2
	; vector2 = vector between target & dino instances
	; vector1 = facing vector from dino head

	lea edx,[ebp-32]
	lea ecx,[esi+CBoundaryBox.OrientationQuat]
	stdcall CONVERT_TO_FACING_PROC_010,edx,0			; get head's facing vector (vector1)

	fld [edi+CInstance.Location.X]
	fsub [esi+CBoundaryBox.Location.X]
	fst DWORD [ebp-20]
	fmul DWORD [ebp-32]
	fld [edi+CInstance.Location.Y]
	fsub [esi+CBoundaryBox.Location.Y]
	fst DWORD [ebp-16]
	fmul DWORD [ebp-28]
	faddp st1,st0
	fstp DWORD [ebp-36]

	; [ebp-32] = vector1
	; [ebp-20] = vector2
	; [ebp-36] = dot product

	fld DWORD [ebp-32]
	fmul st0,st0
	fld DWORD [ebp-28]
	fmul st0,st0
	faddp st1,st0
	fsqrt								; sizeof.vector1

	fld DWORD [ebp-20]
	fmul st0,st0
	fld DWORD [ebp-16]
	fmul st0,st0
	faddp st1,st0
	fsqrt								; sizeof.vector2

	fmulp st1,st0							; sizeof.vector1*sizeof.vector2
	fld [ebx+ANIM_PROJ_INFO.AnimalViewAngleCosine]
	fmulp st1,st0							; sizeof.vector1*sizeof.vector2*cos(b)

	fcomp DWORD [ebp-36]
	fstsw ax
	test ah,41h
	     jz .exit							; FINAL TEST FOR LAUNCH

  ;============================================================
  ; Update launch interval (since launch will surely happen at this point)
  ;============================================================

	mov eax,[ebx+ANIM_PROJ_INFO.ProbabilityInterval_LastTick]
	mov [ebx+ANIM_PROJ_INFO.LastTick],eax				; UPDATE PROJECTILE LAUNCH TICK (only if launch is going to happen)


  ;============================================================
  ; Do projectile launch
  ;============================================================

	mov ecx,ebx
	call GetNextAnimalProjectileInstance
	mov edi,eax

       ; 3 DWORD at [ebp-32] = dino's head's facing UNIT vector (X,Y,Z)
       ; esi = dino head CBoundaryBox instance
       ; ebx = ANIM_PROJ_INFO struct for biting CAnimal
       ; edi = Projectile instance

     ; calculate teleport location for projectile (store at stack_ext_buffer)
	fld DWORD [ebp-32]
	fmul [ebx+ANIM_PROJ_INFO.DistanceFromHead]
	fadd [esi+CBoundaryBox.Location.X]
	fstp DWORD [stack_ext_buffer+0]
	fld DWORD [ebp-28]
	fmul [ebx+ANIM_PROJ_INFO.DistanceFromHead]
	fadd [esi+CBoundaryBox.Location.Y]
	fstp DWORD [stack_ext_buffer+4]
	fld DWORD [ebp-24]
	fmul [ebx+ANIM_PROJ_INFO.DistanceFromHead]
	fadd [esi+CBoundaryBox.Location.Z]
	fstp DWORD [stack_ext_buffer+8]
     ; get orientation for projectile (from head)
	mov eax,[esi+CBoundaryBox.OrientationQuat.W]
	mov edx,[esi+CBoundaryBox.OrientationQuat.X]
	mov DWORD [stack_ext_buffer+12],eax
	mov DWORD [stack_ext_buffer+16],edx
	mov eax,[esi+CBoundaryBox.OrientationQuat.Y]
	mov edx,[esi+CBoundaryBox.OrientationQuat.Z]
	mov DWORD [stack_ext_buffer+20],eax
	mov DWORD [stack_ext_buffer+24],edx

      ; PROJECTILE LOCATION AT stack_ext_buffer (must calculate orientation
      ; that may change if auto-aim enabled before teleporting, and teleport
      ; must be done before velocity set)

      ; CALCULATE PROJECTILE SPEED AND DIRECTION
      ; (if needed, projectile's position at stack_ext_buffer)

	mov ecx,AnimProjAT110Inst

	fld DWORD [ebp-32]
	fmul [ebx+ANIM_PROJ_INFO.ProjectileSpeed]
	fstp [ecx+CAT110.Velocity.X]
	fld DWORD [ebp-28]
	fmul [ebx+ANIM_PROJ_INFO.ProjectileSpeed]
	fstp [ecx+CAT110.Velocity.Y]
	fld DWORD [ebp-24]
	fmul [ebx+ANIM_PROJ_INFO.ProjectileSpeed]
	fstp [ecx+CAT110.Velocity.Z]

      ; CHECK FOR X-Y AUTOAIM
	 cmp [ebx+ANIM_PROJ_INFO.UseHorizontalAutoAim],FALSE
	     jz @f

	  ; MUST KEEP X-Y VECTOR MAGNITUDE SAME AS PREVIOUS
	  ; Ynew/Xnew = Ycurrent/Xcurrent
	  ; MAGNITUDE^2 = Yold^2 + Xold^2        ;Xold and Yold from  [ecx+CSetPhysicsAction.Velocity]
	  ; -> want new magnitude to match old but keep Y/X proportions from current values
	  ;    MAGNITUDE^2 = Ynew^2 + Xnew^2
	  ;    Ynew^2 + Xnew^2 = Yold^2 + Xold^2
	  ;           Ynew = (Ycurrent/Xcurrent)*Xnew
	  ;    ((Ycurrent/Xcurrent)*Xnew)^2 + Xnew^2 = Yold^2 + Xold^2
	  ;    Xnew^2*((Ycurrent/Xcurrent)^2+1) = Yold^2 + Xold^2
	  ;    Xnew = sqrt( (Yold^2 + Xold^2) / ((Ycurrent/Xcurrent)^2+1) )
	  ;         = sqrt( (Yold^2 + Xold^2) / ( Ycurrent^2/Xcurrent^2 + 1) )
	  ;         = sqrt( (Yold^2 + Xold^2) / ( Ycurrent^2/Xcurrent^2 + Xcurrent^2/Xcurrent^2) )
	  ;         = sqrt( (Yold^2 + Xold^2) / ( (Ycurrent^2 + Xcurrent^2)/Xcurrent^2 ) )
	  ;         = sqrt( (Yold^2 + Xold^2)*Xcurrent^2 / (Ycurrent^2 + Xcurrent^2) )
	  ;         = Xcurrent*sqrt(Yold^2 + Xold^2)/sqrt(Ycurrent^2 + Xcurrent^2)
	  ;    Xnew = Xcurrent*OldMagnitude/CurrentMagnitude
	  ; thus, Xnew/Xcurrent = Ynew/Ycurrent = OldMagnitude/CurrentMagnitude
	  ;    and
	  ;    Ynew = Ycurrent*OldMagnitude/CurrentMagnitude
	  ; notes: - Xold and Yold already take into account projectile speed
	  ;        - X-Y current values are simply assumed to be non-zero
	  ;          (which should be true if distances were set properly)

	    ; get biting target instance and get vector between projectile and target
	    ; (should be non-zero if all minimum distances and offset from head distance were set properly)

	     mov edx,[CurrentOnActBiteTargetBiteInstance]
	     mov eax,(stack_ext_buffer+200)

	     fld [edx+CInstance.Location.X]
	     fsub DWORD [stack_ext_buffer+0]
	     fst DWORD [eax]
	     fmul st0,st0
	     fld [edx+CInstance.Location.Y]
	     fsub DWORD [stack_ext_buffer+4]
	     fst DWORD [eax+4]						; X-Ycurrent values at [eax]
	     fmul st0,st0
	     faddp st1,st0
	     fsqrt							; CurrentMagnitude

	     fld [ecx+CAT110.Velocity.X]
	     fmul st0,st0
	     fld [ecx+CAT110.Velocity.Y]
	     fmul st0,st0
	     faddp st1,st0
	     fsqrt							; OldMagnitude

	     fdivrp st1,st0						; OldMagnitude/CurrentMagnitude

	     fld DWORD [eax]
	     fmul st0,st1
	     fstp [ecx+CAT110.Velocity.X]				; Xnew = Xcurrent*OldMagnitude/CurrentMagnitude
	     fmul DWORD [eax+4]
	     fstp [ecx+CAT110.Velocity.Y]				; Ynew = Ycurrent*OldMagnitude/CurrentMagnitude

      @@:

      ; CHECK FOR Z AUTOAIM
      ; -> completely independant from X-Y autoaim, because will destroy
      ;    resulting vector's magnitude (ONLY adds to or subs from Z component for changes)

	 cmp [ebx+ANIM_PROJ_INFO.UseVerticalAutoAim],FALSE
	     jz @f

	   ; get biting target instance
	     mov edx,[CurrentOnActBiteTargetBiteInstance]

	   ; get time that will be taken for projectile to reach target with current
	   ; X-Y speed

	     fld [edx+CInstance.Location.X]
	     fsub DWORD [stack_ext_buffer+0]
	     fmul st0,st0
	     fld [edx+CInstance.Location.Y]
	     fsub DWORD [stack_ext_buffer+4]
	     fmul st0,st0
	     faddp st1,st0
	     fsqrt							; X-Y distance

	     fld [ecx+CAT110.Velocity.X]
	     fmul st0,st0
	     fld [ecx+CAT110.Velocity.Y]
	     fmul st0,st0
	     faddp st1,st0
	     fsqrt							; X-Y speed

	    ; zspeed = height / time
	    ;        = height / distance/speed
	    ;        = height*speed / distance

	     fld [edx+CInstance.Location.Z]
	     fsub DWORD [stack_ext_buffer+8]				; height
	     fadd [ebx+ANIM_PROJ_INFO.ProjectileZSpeedAdjust]		; HEIGHT Z SPEED ADJUST VALUE
	     fmulp st1,st0						; height * speed
	     fdivrp st1,st0						; height*speed / distance
	     fstp [ecx+CAT110.Velocity.Z]
      @@:

      ; SET PROJECTILE'S ORIENTATION from its intended velocity (must be
      ; done before velocity is set because includes teleport)
      ; VELOCITY AT [ecx+CAT110.Velocity._]. Set orientation at 4 DWORD [stack_ext_buffer+12]
      ; This is only required if an Auto Aim setting is used, because otherwise
      ; the projectile will have the same orientation as the head

	mov al,[ebx+ANIM_PROJ_INFO.UseVerticalAutoAim]
	and al,[ebx+ANIM_PROJ_INFO.UseHorizontalAutoAim]
	    jz @f
	    push ecx
	    mov edx,(stack_ext_buffer+12)
	    lea ecx,[ecx+CAT110.Velocity]
	    call [PTRVectorToQuaternion]
	    pop ecx
	@@:

      ; TELEPORT PROJECTILE IN FRONT OF HEAD AND SET ORIENTATION (if no auto-aim,
      ; will be same as head orientation)

	mov ecx,edi
	mov eax,[edi]
	push 0
	push stack_ext_buffer
	call [eax+ICInstance.Teleport]

      ; SET PROJECTILE CONSTANT VELOCITY VIA NEW ACTIONTYPE 110 AND FIXED SPEED CODE

	mov ecx,AnimProjAT110Inst
	mov eax,[edi+CInstance.InstanceNameID]
	mov edx,[ebx+ANIM_PROJ_INFO.ProjectileExtinguishTime]
	mov [ecx+CAT110.InstanceNameID],eax
	mov [ecx+CAT110.Interval],edx
	mov [ecx+CAT110.refCount],0
	mov edx,[ecx]
	mov [ecx+CAT110.ForceBool],TRUE
	mov [ecx+CAT110.SourceInstanceNameID],0
	mov [ecx+CAT110.TotalVelocity],0
	call [edx+ICAction.MainProc]

.exit:
pop ebx
pop esi
pop edi
mov esp,ebp
pop ebp
ret

;============================================================
ATXPROC GetNextAnimalProjectileInstance 	; ecx = ANIM_PROJ_INFO struct
;============================================================
; assumes at least one non-zero projectile instance in struct
; (always true due to context)

push ebx

	mov eax,[ecx+ANIM_PROJ_INFO.NextProjectileIndex]
	lea edx,[ecx+eax+ANIM_PROJ_INFO.ProjectileInstance]
	lea ebx,[ecx+ANIM_PROJ_INFO.ProjectileInstance+(4*8)]

	.getnextprojloop:
	add edx,4
	cmp edx,ebx
	    jb @f
	    lea edx,[ecx+ANIM_PROJ_INFO.ProjectileInstance]
	@@:
	mov eax,[edx]							; projectile instance
	test eax,eax
	     jz .getnextprojloop					; must find a projectile instance

	sub edx,ecx
	sub edx,ANIM_PROJ_INFO.ProjectileInstance
	mov [ecx+ANIM_PROJ_INFO.NextProjectileIndex],edx		; update projectile index for next function call

pop ebx
ret

