;=======================================================
#label GetParticlesObject()_2 = 0x00605870, 0x005D38E0, 0x00608060, 0x006086E0
;=======================================================
OBJECT GetParticlesObject(
 ID32 ParticleNameID32,
 DWORDINT Param2
);

;=======================================================
#label CreateGlobalParticlesObject()_1 = 0x00604F50
;=======================================================

CParticles* CreateGlobalParticlesObject(
 CParticles* particles
); // returns the newly-allocated CParticles instance copy

 This copies the particles instance with dynamic allocation and
 adds the memory to a global list. It does not associate
 the instance with a name; the calling function must pass
 the returned pointer to another function which does.

;=======================================================
#label AssignNameToGlobalParticlesObject()_2 = 0x00605000
;=======================================================

void AssignNameToGlobalParticlesObject(
 ID32 nameHash,
 CParticles* particles
);

 This assigns the hashed string name to the global particles
 objects using a global list. The particles instance should
 have been allocated with CreateGlobalParticlesObject, and
 the nameHash should be created with GetStringID32NoCase.
 Typically, this is used to assign the A00...Axx names
 in T-Scripts to the CParticles group's corresponding object
 after it is constructed.

;=======================================================
; class CParticles (unofficial) and CParticlesLocal (unofficial)
;=======================================================

sizeof.CParticles equ 6Ch

CParticles is the class that defines the instances created whenever
a "CParticles" object is found in a level script upon loading.
It has no VTable. I am assuming it was correspondingly
named CParticles in the source. It is used for the default
CParticles objects the game always initializes internally. There
is one CParticles object for every material, i.e., "Blood", "Terrain", etc.

These are then used in real-time to create temporary instances/copies of themselves
to arrange for displaying their respective effects (the particles
you see in-game). I have labeled these copies CParticlesLocal, which is the same
as CParticles but usually has a few members updated upon copy and creation. They were
possibly defined as mere CParticles in the source, but I make the distinction
here to designate which methods treat their instances as temporary. The default
CParticles instances can simply be seen as "master" objects not used in real-time,
while CParticlesLocal instances are only temporary instances of these.

#label CParticles::CParticles()_3 = 0x00521AE0
#label CParticles::CreateEffectInstance()_8 = 0x00522670
#label CParticlesLocal::InitiateEffect()_7 = 0x005220E0

CParticles CParticles::CParticles(
 UNKNOWN Param1,
 UNKNOWN Param2,
 UNKNOWN Param3
);

 Main constructor. Used to create instance in local
 memory from "CParticles" groups in level files.
 Instance is copied to another location later.

VOID CParticles::CreateEffectInstance(
 SOBJECTLOC EffectLoc	   // 3 params: x,y,z -> location of particle effect
 CVECTOR3 EffectDir	   // 3 params: x,y,z (vector components) -> direction of particle effect (overall effect depends on effect itself - dust isn't much affected by direction)
 FLOAT Param7,		   // Often 1.0. Seems to dictate the "density" of the particle effect (lower = fewer particles)
 FLOAT ThresholdValue	   // **
);

 This creates a temporary copy of the CParticles object (in local memory) with
 a few updated values (most likely having to due with the current
 circumstances in which the particles need to appear)
 and uses it as "this" object to call CParticlesLocal::InitiateEffect().
 It is called upon object collision messages in the CPhysicsSystem and keeps being
 called until the effect disappears, and possibly elsewhere. The temporary instance
 is lost.

 ** This is a value that has been named based on its use in some of the calling code that calls
 this method. The code compares this value against the "Threshold" float script value in this CParticles object,
 and is calculated in real-time, so it should naturally also be a "Threshold" value itself.
 The calling code only calls CParticles::CreateEffectInstance() if the calculated ThresholdValue is
 greater than or equal to the current CParticles object's "Threshold" script value (unmodified from
 the level script). (the script value is the minimum Threshold).

 Note: Particle effect is only _launched_ if its origin (EffectLoc) falls within the camera field-of-view.

VOID CParticlesLocal::InitiateEffect(
 UNKNOWN Param1,
 FLOAT Param2,
 UNKNOWN Param3,
 UNKNOWN Param4,
 FLOAT Param5,
 FLOAT Param6,
 FLOAT Param7
);

 This uses a CParticlesLocal instance, which is simply a (possibly slightly modified) copy
 of a CParticles instance. It creates the appropriate CParticleBase instances (CParticleDust, CParticleBlood, CParticleSolid)
 for the particles display for the effect and stores these instances in a global list of
 some sort for later rendering (it does not call any rendering functions directly). It creates
 one CTexture instance and stores it in CParticlesLocal (which partly explains why this method
 should not take a CParticles default instance, as we do not want to corrupt the original ones which
 hold the data from the level script particles and default particles). The CTexture instance pointer
 and other necessary info is also given to the CParticleBase instances, so the calling code can apparently
 discard the CParticlesLocal instance upon return (hence why I have referred to it as "temporary").

 Usually, more than one CParticleBase instance is created and stored globally by this method,
 although it is for the same effect. They all share the same CTexture pointer and their number
 is calculated and generated by the method (randomly, between the min and max Number values
 for the effect). They appear to represent the individual particles in an effect.
 Not really sure how the texture is managed in very extensive
 particle effects (is the whole thing displayed only on one texture at every frame? or does
 it reuse the same texture and render to it and the texture itself many times? or does it simply find other textures
 to draw onto?).

 * CTexture instance pointer is stored at offset 0x0064 in the CParticlesLocal instance.

;=======================================================
; class CParticleQualities (unofficial)
;=======================================================

sizeof.CParticleQualities equ 28h

Contains Lifespan, Size, Number, etc. particle qualities.
Two  instances are used as sub-structures for the CParticles
class (Min and Max).

#label CParticles::CParticles()_0 = 0x005218A0


CParticleQualities* CParticleQualities::CParticleQualities(
);

 Initializes all members to default values.










